@@ -54,6 +54,7 @@
 #include <libnautilus-extension/nautilus-column-provider.h>
 #include <libnautilus-private/nautilus-column-chooser.h>
 #include <libnautilus-private/nautilus-column-utilities.h>
+#include <libnautilus-private/nautilus-debug-log.h>
 #include <libnautilus-private/nautilus-directory-background.h>
 #include <libnautilus-private/nautilus-dnd.h>
 #include <libnautilus-private/nautilus-file-dnd.h>
@@ -86,8 +87,6 @@ struct FMListViewDetails {
 
 	NautilusTreeViewDragDest *drag_dest;
 
-	GtkTargetList *source_target_list;
-
 	GtkTreePath *double_click_path[2]; /* Both clicks in a double click need to be on the same row */
 
 	GtkTreePath *new_selection_path;   /* Path of the new selection after removing a file */
@@ -142,6 +141,8 @@ static GList *                  default_visible_columns_auto_value;
 static GList *                  default_column_order_auto_value;
 static GdkCursor *              hand_cursor = NULL;
 
+static GtkTargetList *          source_target_list = NULL;
+
 static GList *fm_list_view_get_selection                   (FMDirectoryView   *view);
 static GList *fm_list_view_get_selection_for_file_transfer (FMDirectoryView   *view);
 static void   fm_list_view_set_zoom_level                  (FMListView        *view,
@@ -487,6 +488,17 @@ motion_notify_callback (GtkWidget *widget,
 	}
 
 	if (view->details->drag_button != 0) {
+		if (!source_target_list) {
+			const GtkTargetEntry *drag_types;
+			int n_drag_types;
+
+			fm_list_model_get_drag_types (&drag_types,
+						      &n_drag_types);
+
+			source_target_list = gtk_target_list_new (drag_types,
+								  n_drag_types);
+		}
+
 		if (gtk_drag_check_threshold (widget,
 					      view->details->drag_x,
 					      view->details->drag_y,
@@ -494,7 +506,7 @@ motion_notify_callback (GtkWidget *widget,
 					      event->y)) {
 			context = gtk_drag_begin
 				(widget,
-				 view->details->source_target_list,
+				 source_target_list,
 				 GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK | GDK_ACTION_ASK,
 				 view->details->drag_button,
 				 (GdkEvent*)event);
@@ -774,6 +786,15 @@ row_expanded_callback (GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *pa
 	view = FM_LIST_VIEW (callback_data);
 
 	if (fm_list_model_load_subdirectory (view->details->model, path, &directory)) {
+		char *uri;
+
+		uri = nautilus_directory_get_uri (directory);
+		nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
+				    "list view row expanded window=%p: %s",
+				    fm_directory_view_get_containing_window (FM_DIRECTORY_VIEW (view)),
+				    uri);
+		g_free (uri);
+
 		fm_directory_view_add_subdirectory (FM_DIRECTORY_VIEW (view), directory);
 		
 		if (nautilus_directory_are_all_files_seen (directory)) {
@@ -837,6 +858,7 @@ row_collapsed_callback (GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *p
 	GtkTreeIter parent;
 	struct UnloadDelayData *unload_data;
 	GtkTreeModel *model;
+	char *uri;
 	
 	view = FM_LIST_VIEW (callback_data);
 	model = GTK_TREE_MODEL (view->details->model);
@@ -852,6 +874,14 @@ row_collapsed_callback (GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *p
 				    -1);
 	}
 	
+
+	uri = nautilus_file_get_uri (file);
+	nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
+			    "list view row collapsed window=%p: %s",
+			    fm_directory_view_get_containing_window (FM_DIRECTORY_VIEW (view)),
+			    uri);
+	g_free (uri);
+
 	unload_data = g_new (struct UnloadDelayData, 1);
 	unload_data->view = view;
 	unload_data->file = file;
@@ -864,6 +894,12 @@ row_collapsed_callback (GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *p
 		       unload_data);
 }
 
+static void
+row_activated_callback (GtkTreeView *treeview, GtkTreePath *path, 
+			GtkTreeViewColumn *column, FMListView *view)
+{
+	activate_selected_items (view);
+}
 
 static void
 subdirectory_unloaded_callback (FMListModel *model,
@@ -932,6 +968,36 @@ key_press_callback (GtkWidget *widget, GdkEventKey *event, gpointer callback_dat
 	return handled;
 }
 
+static void
+fm_list_view_reveal_selection (FMDirectoryView *view)
+{
+	GList *selection;
+
+	g_return_if_fail (FM_IS_LIST_VIEW (view));
+
+        selection = fm_directory_view_get_selection (view);
+
+	/* Make sure at least one of the selected items is scrolled into view */
+	if (selection != NULL) {
+		FMListView *list_view;
+		NautilusFile *file;
+		GtkTreeIter iter;
+		GtkTreePath *path;
+		
+		list_view = FM_LIST_VIEW (view);
+		file = selection->data;
+		if (fm_list_model_get_first_iter_for_file (list_view->details->model, file, &iter)) {
+			path = gtk_tree_model_get_path (GTK_TREE_MODEL (list_view->details->model), &iter);
+
+			gtk_tree_view_scroll_to_cell (list_view->details->tree_view, path, NULL, FALSE, 0.0, 0.0);
+			
+			gtk_tree_path_free (path);
+		}
+	}
+
+        nautilus_file_list_free (selection);
+}
+
 static void
 sort_column_changed_callback (GtkTreeSortable *sortable, 
 			      FMListView *view)
@@ -958,6 +1024,9 @@ sort_column_changed_callback (GtkTreeSortable *sortable,
 	reversed_attr = (reversed ? "true" : "false");
 	nautilus_file_set_metadata (file, NAUTILUS_METADATA_KEY_LIST_VIEW_SORT_REVERSED,
 				    default_reversed_attr, reversed_attr);
+				    
+	/* Make sure selected item(s) is visible after sort */
+	fm_list_view_reveal_selection (FM_DIRECTORY_VIEW (view));							      
 }
 
 static void
@@ -1043,11 +1112,11 @@ get_file_for_path_callback (NautilusTreeViewDragDest *dest,
 
 /* Handles an URL received from Mozilla */
 static void
-list_view_handle_url (NautilusTreeViewDragDest *dest, const char *encoded_url,
-		      const char *target_uri, GdkDragAction action, int x, int y, FMListView *view)
+list_view_handle_netscape_url (NautilusTreeViewDragDest *dest, const char *encoded_url,
+			       const char *target_uri, GdkDragAction action, int x, int y, FMListView *view)
 {
-	fm_directory_view_handle_url_drop (FM_DIRECTORY_VIEW (view),
-					   encoded_url, target_uri, action, x, y);
+	fm_directory_view_handle_netscape_url_drop (FM_DIRECTORY_VIEW (view),
+						    encoded_url, target_uri, action, x, y);
 }
 
 static void
@@ -1182,10 +1251,8 @@ create_and_set_up_tree_view (FMListView *view)
 {
 	GtkCellRenderer *cell;
 	GtkTreeViewColumn *column;
-	const GtkTargetEntry *drag_types;
 	GtkBindingSet *binding_set;
 	AtkObject *atk_obj;
-	int num_drag_types;	
 	GList *nautilus_columns;
 	GList *l;
 	
@@ -1200,8 +1267,6 @@ create_and_set_up_tree_view (FMListView *view)
 	binding_set = gtk_binding_set_by_class (GTK_WIDGET_GET_CLASS (view->details->tree_view));
 	gtk_binding_entry_clear (binding_set, GDK_BackSpace, 0);
 
-	fm_list_model_get_drag_types (&drag_types, &num_drag_types);
-	
 	view->details->drag_dest = 
 		nautilus_tree_view_drag_dest_new (view->details->tree_view);
 
@@ -1217,8 +1282,8 @@ create_and_set_up_tree_view (FMListView *view)
 				 "move_copy_items",
 				 G_CALLBACK (move_copy_items_callback),
 				 view, 0);
-	g_signal_connect_object (view->details->drag_dest, "handle_url",
-				 G_CALLBACK (list_view_handle_url), view, 0);
+	g_signal_connect_object (view->details->drag_dest, "handle_netscape_url",
+				 G_CALLBACK (list_view_handle_netscape_url), view, 0);
 	g_signal_connect_object (view->details->drag_dest, "handle_uri_list",
 				 G_CALLBACK (list_view_handle_uri_list), view, 0);
 	g_signal_connect_object (view->details->drag_dest, "handle_text",
@@ -1250,6 +1315,8 @@ create_and_set_up_tree_view (FMListView *view)
                                  G_CALLBACK (row_expanded_callback), view, 0);
 	g_signal_connect_object (view->details->tree_view, "row_collapsed",
                                  G_CALLBACK (row_collapsed_callback), view, 0);
+	g_signal_connect_object (view->details->tree_view, "row-activated",
+                                 G_CALLBACK (row_activated_callback), view, 0);
 	
 	view->details->model = g_object_new (FM_TYPE_LIST_MODEL, NULL);
 	gtk_tree_view_set_model (view->details->tree_view, GTK_TREE_MODEL (view->details->model));
@@ -1263,10 +1330,6 @@ create_and_set_up_tree_view (FMListView *view)
 	g_signal_connect_object (view->details->model, "subdirectory_unloaded",
 				 G_CALLBACK (subdirectory_unloaded_callback), view, 0);
 
-	view->details->source_target_list = 
-		gtk_target_list_new (drag_types, num_drag_types);
-	
-
 	gtk_tree_selection_set_mode (gtk_tree_view_get_selection (view->details->tree_view), GTK_SELECTION_MULTIPLE);
 	gtk_tree_view_set_rules_hint (view->details->tree_view, TRUE);
 
@@ -1297,6 +1360,7 @@ create_and_set_up_tree_view (FMListView *view)
 			view->details->pixbuf_cell = (GtkCellRendererPixbuf *)cell;
 			
 			view->details->file_name_column = gtk_tree_view_column_new ();
+                        gtk_tree_view_column_set_expand (view->details->file_name_column, TRUE);
 			g_object_ref (view->details->file_name_column);
 			gtk_object_sink (GTK_OBJECT (view->details->file_name_column));
 			view->details->file_name_column_num = column_num;
@@ -1319,6 +1383,10 @@ create_and_set_up_tree_view (FMListView *view)
 							     NULL);
 			
 			cell = gtk_cell_renderer_text_new ();
+                        g_object_set (cell,
+                                      "ellipsize", PANGO_ELLIPSIZE_END,
+                                      "ellipsize-set", TRUE,
+                                      NULL);
 			view->details->file_name_cell = (GtkCellRendererText *)cell;
 			g_signal_connect (cell, "edited", G_CALLBACK (cell_renderer_edited), view);
 			g_signal_connect (cell, "editing-canceled", G_CALLBACK (cell_renderer_editing_canceled), view);
@@ -1452,7 +1520,7 @@ set_sort_order_from_metadata_and_preferences (FMListView *list_view)
 
 	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (list_view->details->model),
 					      sort_column_id,
-					      sort_reversed ? GTK_SORT_DESCENDING : GTK_SORT_ASCENDING);
+					      sort_reversed ? GTK_SORT_DESCENDING : GTK_SORT_ASCENDING);					      
 }
 
 static gboolean
@@ -1806,36 +1874,6 @@ fm_list_view_select_all (FMDirectoryView *view)
 	gtk_tree_selection_select_all (gtk_tree_view_get_selection (FM_LIST_VIEW (view)->details->tree_view));
 }
 
-static void
-fm_list_view_reveal_selection (FMDirectoryView *view)
-{
-	GList *selection;
-
-	g_return_if_fail (FM_IS_LIST_VIEW (view));
-
-        selection = fm_directory_view_get_selection (view);
-
-	/* Make sure at least one of the selected items is scrolled into view */
-	if (selection != NULL) {
-		FMListView *list_view;
-		NautilusFile *file;
-		GtkTreeIter iter;
-		GtkTreePath *path;
-		
-		list_view = FM_LIST_VIEW (view);
-		file = selection->data;
-		if (fm_list_model_get_first_iter_for_file (list_view->details->model, file, &iter)) {
-			path = gtk_tree_model_get_path (GTK_TREE_MODEL (list_view->details->model), &iter);
-
-			gtk_tree_view_scroll_to_cell (list_view->details->tree_view, path, NULL, FALSE, 0.0, 0.0);
-			
-			gtk_tree_path_free (path);
-		}
-	}
-
-        nautilus_file_list_free (selection);
-}
-
 static void
 column_editor_response_callback (GtkWidget *dialog, 
 				 int response_id,
@@ -2453,8 +2491,6 @@ fm_list_view_finalize (GObject *object)
 	if (list_view->details->new_selection_path) {
 		gtk_tree_path_free (list_view->details->new_selection_path);
 	}
-
-	gtk_target_list_unref (list_view->details->source_target_list);
 	
 	g_list_free (list_view->details->cells);
 	g_hash_table_destroy (list_view->details->columns);
