@@ -45,6 +45,7 @@
 #include <libgnome/gnome-macros.h>
 #include <glib/gi18n.h>
 #include <libgnomeui/gnome-popup-menu.h>
+#include <libnautilus-private/nautilus-debug-log.h>
 #include <libnautilus-private/nautilus-dnd.h>
 #include <libnautilus-private/nautilus-bookmark.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
@@ -232,7 +233,7 @@ update_places (NautilusPlacesSidebar *sidebar)
 	if (strcmp (g_get_home_dir(), desktop_path) != 0) {
 		char *display_name;
 
-		mount_uri = gnome_vfs_get_uri_from_local_path (g_get_home_dir ());
+		mount_uri = nautilus_get_home_directory_uri ();
 		display_name = g_filename_display_basename (g_get_home_dir ());
 		last_iter = add_place (sidebar, PLACES_BUILT_IN,
 				       display_name, "gnome-fs-home",
@@ -474,6 +475,9 @@ row_activated_callback (GtkTreeView *tree_view,
 		(model, &iter, PLACES_SIDEBAR_COLUMN_URI, &uri, -1);
 	
 	if (uri != NULL) {
+		nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
+				    "activate from places sidebar window=%p: %s",
+				    sidebar->window, uri);
 		/* Navigate to the clicked location. */
 		nautilus_window_info_open_location
 			(sidebar->window, 
@@ -484,6 +488,14 @@ row_activated_callback (GtkTreeView *tree_view,
 		gtk_tree_model_get 
 			(model, &iter, PLACES_SIDEBAR_COLUMN_DRIVE, &drive, -1);
 		if (drive != NULL) {
+			char *path;
+
+			path = gnome_vfs_drive_get_device_path (drive);
+			nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
+					    "activate drive from places sidebar window=%p: %s",
+					    sidebar->window, path);
+			g_free (path);
+
 			gnome_vfs_drive_mount (drive, volume_op_callback, sidebar);
 			gnome_vfs_drive_unref (drive);
 		}
@@ -1264,9 +1276,11 @@ unmount_shortcut_cb (GtkMenuItem           *item,
 			    -1);
 
 	if (volume != NULL) {
-		gnome_vfs_volume_unmount (volume, volume_op_callback, sidebar);
+		nautilus_file_operations_unmount_volume (GTK_WIDGET (sidebar->tree_view),
+				volume, volume_op_callback, sidebar);
 	} else if (drive != NULL) {
-		gnome_vfs_drive_unmount (drive, volume_op_callback, sidebar);
+		nautilus_file_operations_unmount_drive (GTK_WIDGET (sidebar->tree_view),
+				drive, volume_op_callback, sidebar);
 	}
 	gnome_vfs_volume_unref (volume);
 	gnome_vfs_drive_unref (drive);
